/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Sortov (sortov@ispras.ru)
// SYNOPSIS: Support of configuration files
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <string>
#include <map>

#ifndef uint
typedef unsigned int uint;
#endif

namespace cpptesk {
namespace ts {

#define CPPTESK_DECLARE_OPTION(type, name, default_value) \
    const static type default_ ## name = default_value; \
                 type             name;

#if defined(CPPTESK_SHORT_NAMES)
#define DECLARE_OPTION CPPTESK_DECLARE_OPTION
#endif

#if defined(CPPTESK_SHORT_SHORT_NAMES)
#define DECLARE_OPTION CPPTESK_DECLARE_OPTION
#endif


#define CPPTESK_ADD_OPTION(option) add_option(option, #option, default_ ## option)

#if defined(CPPTESK_SHORT_NAMES)
#define ADD_OPTION CPPTESK_ADD_OPTION
#endif

#if defined(CPPTESK_SHORT_SHORT_NAMES)
#define ADD_OPTION CPPTESK_ADD_OPTION
#endif

class Option {
public:
    std::string type;
    void        *ptr;

    Option() {}
    Option(const std::string type, void *ptr): type(type), ptr(ptr) {}
};

class Config
{
    public:
        Config();
        virtual ~Config();

        void add_option(bool  &option, const std::string &name, bool  default_value);
        void add_option(int   &option, const std::string &name, int   default_value);
        void add_option(uint  &option, const std::string &name, uint  default_value);
        void add_option(float &option, const std::string &name, float default_value);

        void parse(const std::string &filename);

    private:
        void add_option(const std::string &name, const std::string &type, void *ptr);

        std::map<std::string, Option> options;
};

}} // cpptesk::ts
