/** @file fifo_scen.cpp
 * Scenarios of Multififo verification.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#include <fifo_scen.h>

namespace cpptesk {
namespace examples {
namespace fifo {

bool FIFOScenario::init(int argc, char ** argv)
{
    return true;
}

void FIFOScenario::finish()
{
    dut.finalize();
}

bool FIFOScenario::scen_nop(Ctx& ctx)
{
    CPPTESK_ITERATION_BEGIN
    CPPTESK_ITERATION_ACTION
    {
        dut.cycle();
        CPPTESK_ITERATION_YIELD(dut.verdict());
    }
    CPPTESK_ITERATION_END
}

bool FIFOScenario::scen_push_a(Ctx& ctx)
{
    CPPTESK_ITERATION_BEGIN
    CPPTESK_ITERATION_ACTION
    {
        if(dut.isIface1aReady() && !dut.fifo0.is_full() && !dut.fifo2.is_full()) {
            InputData msg = InputData();
            CPPTESK_CALL_STIMULUS_OF(dut, Multififo::push_msg_a, dut.iface1a, msg);
        }
        CPPTESK_ITERATION_YIELD(dut.verdict());
    }
    CPPTESK_ITERATION_END
}

bool FIFOScenario::scen_push_b(Ctx& ctx)
{
    CPPTESK_ITERATION_BEGIN
    CPPTESK_ITERATION_ACTION
    {
        if(dut.isIface1bReady() && !dut.fifo1.is_full() && !dut.fifo2.is_full()) {
            InputData msg = InputData();
            CPPTESK_CALL_STIMULUS_OF(dut, Multififo::push_msg_b, dut.iface1b, msg);
        }
        CPPTESK_ITERATION_YIELD(dut.verdict());
    }
    CPPTESK_ITERATION_END
}

bool FIFOScenario::scen_pop(cpptesk::ts::IntAbcCtx& ctx)
{
    CPPTESK_ITERATION_BEGIN
    CPPTESK_ITERATION_ACTION
    {
        if(dut.isIface2Ready() && !dut.fifo2.is_empty()) {
            InputData msg = InputData();
            CPPTESK_CALL_STIMULUS_OF(dut, Multififo::pop_msg, dut.iface2, msg);
        }
        CPPTESK_ITERATION_YIELD(dut.verdict());
    }
    CPPTESK_ITERATION_END
}

std::string FIFOScenario::get_state_fifo()
{
    std::string state;
    std::stringstream out;

    out << "{fifo:" << dut.fifo0.fifo_fulness() << "," << dut.fifo1.fifo_fulness() << "," << dut.fifo2.fifo_fulness() 
        << ", write1_started:" << dut.isIface1aReady() << ", write2_started:" << dut.isIface1bReady() << ", read_started:" << dut.isIface2Ready() << "}";
    state = out.str();
    CPPTESK_DEBUG_PRINT(cpptesk::hw::DEBUG_USER, "get_state_fifo(): " << state);
    return state;
}

FIFOScenario::FIFOScenario(void) : ScenarioBase()
{
}
FIFOScenario::~FIFOScenario(void) {}

ExampleScenario::ExampleScenario(void) : FIFOScenario()
{
    setup("MultiFIFOScenario", &FIFOScenario::init, &FIFOScenario::finish, &FIFOScenario::get_state_fifo);
    CPPTESK_ADD_SCENARIO_METHOD(FIFOScenario::scen_nop);
    CPPTESK_ADD_SCENARIO_METHOD(FIFOScenario::scen_push_a);
    CPPTESK_ADD_SCENARIO_METHOD(FIFOScenario::scen_push_b);
    CPPTESK_ADD_SCENARIO_METHOD(FIFOScenario::scen_pop);
}
ExampleScenario::~ExampleScenario(void) { };

}}}

