/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Diagnostics subsystem main file
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/event.hpp>
#include <hw/interface.hpp> 
#include <hw/sim_info.hpp>

#include <hw/diagnostics/error_model.hpp>
#include <hw/diagnostics/reaction_unit_vector.hpp>
#include <hw/diagnostics/reaction_pair_vector.hpp>

#include <hw/report/model.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

struct ReportUnit
{
    cpptesk::hw::report::Report report;
    ReactionUnitVector reaction_vector;
};

    
class Diagnostics
{
public:
    Diagnostics(const event_list_t &reactions, const SimulationInfo &sim_info);

    virtual ~Diagnostics() {}

    void apply_normal_rules();
    void apply_fuzzy_rules();
    
    /// Convert optional reactions to error reactions
    void optional_conversion();
    
    /// Start the diagnostics subsystem
     ReportUnit run();

protected:
    /// List of rules
    ErrorModel _error_model;

    /// List of reaction pairs
    ReactionUnitVector _reaction_units;

    /// Additional information from simulation stage
    SimulationInfo _simulation_info;
};

}}}
