/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Vector of Reaction Units
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <vector>

#include <hw/message.hpp>
#include <hw/diagnostics/reaction_unit.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

class ReactionUnitVector
{
public:
    ReactionUnitVector();
    ~ReactionUnitVector();

    ReactionUnitVector(cpptesk::hw::Reaction &r1, cpptesk::hw::Reaction &r2,
                int parent_r1_spec_id, int parent_r1_impl_id,
                int parent_r2_spec_id, int parent_r2_impl_id);

    ReactionUnitVector(cpptesk::hw::Reaction &r,
                int parent_spec_id, int parent_impl_id);

    ReactionUnit& operator[] (uint index);

    uint size() const;
    bool is_empty() const;

    void add_reaction_unit(ReactionUnit ru);

    void add_reaction(const cpptesk::hw::Reaction &reaction, int parent_spec, int parent_impl);
    void add_reaction(const cpptesk::hw::Reaction &reaction);

    void add_reaction_units(ReactionUnitVector &rs);

    void init(const event_list_t &reactions);

protected:
    std::vector<ReactionUnit> _reaction_vector;
};

}}}
