/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Point event is a triple <message, interface, time>.
//           Prolonged event has an additional time dimension (duration).
///////////////////////////////////////////////////////////////////////////////

#include <hw/event.hpp>
#include <hw/interface.hpp>

namespace cpptesk {
namespace hw {

bool PointEvent::isInput() const
{
    return _iface && _iface->isInput();
}

bool PointEvent::isOutput() const
{
    return _iface && _iface->isOutput();
}

std::string Reaction::getShortDescription() const
{
    std::stringstream buffer;

    buffer << getErrorString(_error);
    buffer << "(";
    {
        buffer << *_iface << ":" << std::dec;

        if(_start_time == _stop_time)
            { buffer << _start_time; }
        else
            { buffer << "[" << _start_time << ".." << _stop_time << "]"; }
    }
    buffer << ")";

    return buffer.str();
}

std::string Reaction::getFullDescription() const
{
    std::stringstream buffer;

    if(_info.empty())
        { return getShortDescription(); }

    buffer << getShortDescription() << ": " << _info;
    return buffer.str();
}

}} // namespace cpptesk::hw

