/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Report printer
// ///////////////////////////////////////////////////////////////////////////////

#include <hw/report/printer.hpp>

namespace cpptesk {
namespace hw {
namespace report {

void DefaultPrinter::print(std::ostream &out, const Report &report)
{
    const Failures &failures = report.failures;

    out << "================================================================================" << std::endl;
    out << failures.name << std::endl;
    out << "================================================================================" << std::endl;

    for(cpptesk::hw::report::Failures::const_iterator i = failures.begin(); i != failures.end(); i++)
    {
        const cpptesk::hw::report::FailureGroup &failure_group = *i;
    
        const cpptesk::hw::report::Failure &failure = *failure_group.begin();

        out << "--------------------------------------------------------------------------------" << std::endl;
        out << failure.name        << std::endl;
        out << "--------------------------------------------------------------------------------" << std::endl;

        out << failure.type    << std::endl;
        out << failure.value   << std::endl;
        out << failure.unit    << std::endl;
        out << failure.info    << std::endl;
        out << failure.failure << std::endl;

        for(cpptesk::hw::report::DataFailure::const_iterator j = failure.data.begin(); j != failure.data.end(); j++)
        {
            const cpptesk::hw::report::FieldFailure &field = *j;

            FieldList field_impl = field.impl;
            out << "Impl = ";
            for(cpptesk::hw::report::FieldList::const_iterator i = field_impl.begin(); i != field_impl.end(); i++)
            {
                const cpptesk::hw::report::ItemField &item = *i;
                
                out << item.value;
            }   
            out << std::endl;

            FieldList field_spec = field.impl;
            out << "Spec = ";
            for(cpptesk::hw::report::FieldList::const_iterator i = field_spec.begin(); i != field_spec.end(); i++)
            {
                const cpptesk::hw::report::ItemField &item = *i;
                
                out << item.value;
            }   
            out << std::endl;
        }  
        
        const TimeFailure &time = failure.time;

        out << "--------------------------------------------------------------------------------" << std::endl;
        out << time.name    << std::endl;
        out << "--------------------------------------------------------------------------------" << std::endl;

        out << time.spec    << std::endl;
        out << time.impl    << std::endl;
        out << time.failure << std::endl;

        const IfaceFailure &iface = failure.iface;

        out << "--------------------------------------------------------------------------------" << std::endl;
        out << iface.name    << std::endl;
        out << "--------------------------------------------------------------------------------" << std::endl;

        out << iface.spec    << std::endl;
        out << iface.impl    << std::endl;
        out << iface.failure << std::endl;
    }

    const Simulation &simulation = report.simulation;

    out << "================================================================================" << std::endl;
    out << simulation.name << std::endl;
    out << "================================================================================" << std::endl;

    out << simulation.cycle.name << " = " << simulation.cycle.value << " " << simulation.cycle.unit << std::endl;
    out << simulation.time.name << " = " << simulation.time.value << " " << simulation.time.unit << std::endl;
    out << simulation.frequency.name << " = " << simulation.frequency.value << " " << simulation.frequency.unit << std::endl;
        
    const Statistics &statistics = report.statistics;

    out << "================================================================================" << std::endl;
    out << statistics.name << " = " << statistics.value << " " << statistics.unit << std::endl;
    out << "================================================================================" << std::endl;

    out << statistics.stimuli.name << " = " << statistics.stimuli.value << std::endl;
    out << statistics.reactions.name << " = " << statistics.reactions.value << std::endl;
    out << statistics.normal.name << " = " << statistics.normal.value << std::endl;
    out << statistics.incorrect.name << " = " << statistics.incorrect.value << std::endl;
    out << statistics.missing.name << " = " << statistics.missing.value << std::endl;
    out << statistics.unexpected.name << " = " << statistics.unexpected.value << std::endl;
}

}}} // namespace cpptesk::hw::report

