/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Event manager 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/sequence/event.hpp>

#include <set>
#include <string>

namespace cpptesk {
namespace hw {
namespace sequence {

////////////////////////////////////////////////////////////////////////////////
// Model Event
////////////////////////////////////////////////////////////////////////////////

class ModelEventManager;

class ModelEvent: public Event
{
public:
    ModelEvent(const std::string &id, const ModelEventManager &manager):
        _id(id), _manager(manager) {}
    ModelEvent(const ModelEvent &rhs): Event(),
        _id(rhs._id), _manager(rhs._manager) {}

    virtual ~ModelEvent() {}

    virtual ModelEvent* clone() const { return new ModelEvent(*this); }

    virtual bool occured() const;

    const std::string& id() const { return _id; }

protected:
    /// Event identifier (unique string).
    std::string _id;
    /// Reference to the model event manager.
    const ModelEventManager &_manager;
};

////////////////////////////////////////////////////////////////////////////////
// Model Event Manager
////////////////////////////////////////////////////////////////////////////////

class ModelEventManager
{
public:
    ModelEventManager() {}
    virtual ~ModelEventManager() {}

    void add(const ModelEvent &event) { _events.insert(event.id()); }

    bool contains(const ModelEvent &event) const { return _events.find(event.id()) != _events.end(); }

    size_t size() const { return _events.size(); }

    void cycle() { _events.clear(); }

protected:
    std::set<std::string> _events;
};

}}} // namespace cpptesk::hw::sequence

