/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Test sequence tracer 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/time.hpp>
#include <hw/utils/singleton.hpp>

#include <string>
#include <fstream>

#define CPPTESK_SET_TRACE_FILE(file_name) \
    ::cpptesk::hw::Tracer::get().setTraceFile(file_name)

#ifdef HW_SHORT_NAMES
    #define SET_TRACE_FILE(file_name) CPPTESK_SET_TRACE_FILE(file_name)
#endif // HW_SHORT_NAMES

namespace cpptesk {
namespace hw {

class Message;

class Tracer: public Singleton<Tracer>
{
public:
    friend class Singleton<Tracer>;

    virtual ~Tracer();

    void setTraceFile(const char *filename);

    bool isEnabled() const { return _test_sequence_stream != 0; }

    void trace
    (
        cycle_t time,
        const std::string &object_name,
        const std::string &stimulus_name,
        const std::string &iface_name,
        const std::string &message_name,
        const Message &message_itself
    );

private:
    Tracer(): _latest_record_time(0), _test_sequence_stream(0) {}

    cycle_t _latest_record_time;
    std::ofstream *_test_sequence_stream;
};

}} // cpptesk::hw

