/*
 * Copyright 2010-2012 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Veritool-based mediator of a hardware model 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/media.hpp>
#include <hw/veritool/synchronizer.hpp>

////////////////////////////////////////////////////////////////////////////////
// Declaration of Mediator Classes
////////////////////////////////////////////////////////////////////////////////

/** @def CPPTESK_VERITOOL_MEDIA
 * Declares a Veritool-based mediator (adapter) of a hardware model.
 */
#define CPPTESK_VERITOOL_MEDIA(UserMedia, UserModel) \
    CPPTESK_SIGNAL_MEDIA(UserModel##_VeritoolMedia, UserModel, inputs_t, outputs_t) { \
        UserModel##_VeritoolMedia() { setSynchronizer(new ::cpptesk::hw::veritool::VeritoolSynchronizer<inputs_t, outputs_t>(*this)); } \
    }; \
    CPPTESK_MEDIA(UserMedia, UserModel##_VeritoolMedia)

/** @def CPPTESK_VERITOOL_ADAPTER
 * Alias of CPPTESK_VERITOOL_MEDIA
 */
#define CPPTESK_VERITOOL_ADAPTER(UserMedia, UserModel) CPPTESK_VERITOOL_MEDIA(UserMedia, UserModel)

#ifdef HW_SHORT_NAMES
    #define VERITOOL_MEDIA(UserMedia, UserModel)   CPPTESK_VERITOOL_MEDIA(UserMedia, UserModel)
    #define VERITOOL_ADAPTER(UserMedia, UserModel) CPPTESK_VERITOOL_ADAPTER(UserMedia, UserModel)
#endif // HW_SHORT_NAMES


