/** @file counter_scen.cpp
 * This file contains implementation of the scenario class.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#include <counter_scen.h>

namespace cpptesk {
namespace examples {
namespace counter {

bool CounterScenario::init(int argc, char ** argv)
{
    UNUSED_ARG(argc);
    UNUSED_ARG(argv);

    dut.initialize();
#ifdef CPPTESK_AUTOFSM
    dut.delay(5);
    autofsm.set_show_start(false);
    autofsm.set_show_cycle(false);
    autofsm.set_show_guard(false);
    autofsm.initialize();
#endif

    return true;
}

void CounterScenario::finish()
{
    dut.finalize();
}

#ifdef CPPTESK_AUTOFSM
bool CounterScenario::scen_nop(Ctx& ctx)
{
    ITERATION_BEGIN
    ITERATION_ACTION
    {
        autofsm.scen_nop();
        ITERATION_YIELD(dut.verdict());
    }
    ITERATION_END
}
#endif

bool CounterScenario::scen_inc(Ctx& ctx)
{
    InputData msg;

    ITERATION_BEGIN

    ITERATION_ACTION
    {
        dut.refresh_coverage();

#ifdef CPPTESK_AUTOFSM
        if(dut.isIface1Ready())
        {
#endif
            msg = InputData();
            CPPTESK_CALL_STIMULUS_OF(dut, Counter::inc, dut.iface1, msg);
#ifdef CPPTESK_AUTOFSM
        }
#else
        dut.cycle();
#endif

        ITERATION_YIELD(dut.verdict());
    }
    ITERATION_END
}

std::string CounterScenario::get_state_counter()
{
#ifdef CPPTESK_AUTOFSM
    return autofsm.get_state();
#else
    std::string state;
    std::stringstream out;

    out << "{counter:" << dut.get_value() << "}";
    state = out.str();

    DEBUG_PRINTF(cpptesk::hw::DEBUG_USER, "get_state_counter(): %s", state.c_str());

    return state;
#endif
}

CounterScenario::CounterScenario(void) : ScenarioBase()
#ifdef CPPTESK_AUTOFSM
, autofsm(dut)
#endif
{
}
CounterScenario::~CounterScenario(void) {}

ExampleScenario::ExampleScenario(void) : CounterScenario()
{
    setup("CounterScenario", &CounterScenario::init, &CounterScenario::finish, &CounterScenario::get_state_counter);

#ifdef CPPTESK_AUTOFSM
    ADD_SCENARIO_METHOD(CounterScenario::scen_nop);
#endif
    ADD_SCENARIO_METHOD(CounterScenario::scen_inc);
}
ExampleScenario::~ExampleScenario(void) { };

}}}

