/** @file counter_model.cpp
 * This file contains implementation of the counter model class.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#include <counter_model.h>

namespace cpptesk {
namespace examples {
namespace counter {

Counter::Counter() : coverage(*this)
{
    ADD_INPUT(iface1);
    ADD_COMPONENT(iface2);
    memset(&refinputs, 0, sizeof(refinputs));
    memset(&refoutputs, 0, sizeof(refoutputs));
    memset(&refreqs, 0, sizeof(refreqs));
    memset(refcurrentstate, 0, sizeof(refcurrentstate));
}

Counter::~Counter() {}

bool Counter::isIface1Ready() const
{
    return iface1.isReady();
}

// Reaction checking answer of ref model
DEFINE_PROCESS(Counter::answer_from_ref_model)
{
    START_PROCESS();
    OutputData &data = CAST_MESSAGE(OutputData);
    UNUSED_ARG(iface);

    int counter = 0;
    while(!(refoutputs.valid == 1 && refreqs.out_id == data.get_id() && refreqs.out_timemark == data.get_timemark()))
    {
        CYCLE();
        ASSERTION(++counter < 200, "too long waiting for refmodel answer");
    }

    data.set_data(refoutputs.data);

    STOP_PROCESS();
}

DEFINE_PROCESS(Counter::inc_send_to_ref_model)
{
    InputData data = CAST_MESSAGE(InputData);
    UNUSED_ARG(iface);

    START_PROCESS();

    refinputs.valid = 1;
    refreqs.in_id = data.get_id();
    refreqs.in_timemark = data.get_timemark();

    CYCLE();
    STOP_PROCESS();
}

DEFINE_STIMULUS(Counter::inc)
{
    InputData data = CAST_MESSAGE(InputData);
    OutputData outdata = OutputData(data.get_id(), data.get_timemark());

    START_STIMULUS(PARALLEL);

    CALL_PROCESS(PARALLEL, Counter::inc_send_to_ref_model, GET_IFACE(), data);
    CALL_PROCESS(SEQUENTIAL, Counter::answer_from_ref_model, GET_IFACE(), outdata);
    CYCLE();
    CALL_PROCESS(PARALLEL, Counter::read_result, iface2, outdata);

    CYCLE();
    STOP_STIMULUS();
}

DEFINE_REACTION(Counter::read_result)
{
    START_PROCESS();

    SEND_REACTION(PARALLEL, GET_IFACE(), GET_MESSAGE());

    CYCLE();
    STOP_PROCESS();
}

void Counter::refresh_coverage(void)
{
    coverage.out_cov();
}

/*
 * Connection to SystemVerilog model
 */
InputSignals refinputs;
OutputSignals refoutputs;
Requests refreqs;
char refcurrentstate[255];

extern "C" {
    void init_state(char *state)
    {
        strncpy(refcurrentstate, state, 255);
    }

    void take_inputs(InputSignals *inputs, Requests *requests)
    {
        *inputs = refinputs;
        *requests = refreqs;
        memset(&refinputs, 0, sizeof(refinputs));
        memset(&refreqs, 0, sizeof(refreqs));
    }

    void send_outputs(OutputSignals *outputs, Requests *requests)
    {
        refoutputs = *outputs;
        refreqs = *requests;
    }
}

}}}
